@@ -22,7 +22,7 @@
 
 #ifdef HAVE_TIMERFD
 #include <sys/timerfd.h>
-#endif
+#endif /*HAVE_TIMERFD */
 
 #include <errno.h>
 #include <unistd.h>
@@ -81,11 +81,11 @@ goa_alarm_dispose (GObject *object)
   GoaAlarm *self = GOA_ALARM (object);
 
   g_clear_object (&self->priv->stream);
-  g_clear_pointer (&self->priv->immediate_wakeup_source, (GDestroyNotify) g_source_destroy);
-  g_clear_pointer (&self->priv->scheduled_wakeup_source, (GDestroyNotify) g_source_destroy);
-  g_clear_pointer (&self->priv->context, (GDestroyNotify) g_main_context_unref);
-  g_clear_pointer (&self->priv->time, (GDestroyNotify) g_date_time_unref);
-  g_clear_pointer (&self->priv->previous_wakeup_time, (GDestroyNotify) g_date_time_unref);
+  g_clear_pointer (&self->priv->immediate_wakeup_source, g_source_destroy);
+  g_clear_pointer (&self->priv->scheduled_wakeup_source, g_source_destroy);
+  g_clear_pointer (&self->priv->context, g_main_context_unref);
+  g_clear_pointer (&self->priv->time, g_date_time_unref);
+  g_clear_pointer (&self->priv->previous_wakeup_time, g_date_time_unref);
 
   G_OBJECT_CLASS (goa_alarm_parent_class)->dispose (object);
 }
@@ -281,14 +281,10 @@ on_timer_source_ready (GObject *stream, GoaAlarm *self)
 
   if (bytes_read < 0)
     {
-      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-        g_debug ("GoaAlarm: discontinuity detected from timer fd");
-      else
-        {
-          g_warning ("GoaAlarm: failed to read from timer fd: %s\n",
-                     error->message);
-          goto out;
-        }
+      g_warning ("GoaAlarm: failed to read from timer fd: %s\n",
+                 error->message);
+      g_error_free (error);
+      goto out;
     }
 
   if (bytes_read == sizeof (gint64))
@@ -304,10 +300,9 @@ on_timer_source_ready (GObject *stream, GoaAlarm *self)
   run_again = TRUE;
 out:
   g_rec_mutex_unlock (&self->priv->lock);
-  g_clear_error (&error);
   return run_again;
 }
-#endif
+#endif /*HAVE_TIMERFD */
 
 static gboolean
 schedule_wakeups_with_timerfd (GoaAlarm *self)
