@@ -100,8 +100,8 @@ goa_kerberos_identity_inquiry_dispose (GObject *object)
   GoaKerberosIdentityInquiry *self = GOA_KERBEROS_IDENTITY_INQUIRY (object);
 
   g_clear_object (&self->priv->identity);
-  g_clear_pointer (&self->priv->name, (GDestroyNotify) g_free);
-  g_clear_pointer (&self->priv->banner, (GDestroyNotify) g_free);
+  g_clear_pointer (&self->priv->name, g_free);
+  g_clear_pointer (&self->priv->banner, g_free);
 
   G_OBJECT_CLASS (goa_kerberos_identity_inquiry_parent_class)->dispose (object);
 }
